#ifndef PROPELLER
#define PROPELLER

#ifdef __cplusplus
extern "C" 
{
#endif

#include <stdint.h>

#define hub_memory_base         0x10000000
#define hub_memory_size         32768
#define rendezvous_area_size    128
#define vm_mbox_size            (2 * sizeof(uint32_t))
#define vm_params_size          (4 * sizeof(uint32_t))
#define serial_data_size        sizeof(FdSerial_t)
#define sector_size             512
#define scratch_buffer_size     2048    // big enough for a sector or a COG memory image
#define vm_cache_size           8192

#define rendezvous_area_off     (hub_memory_size - rendezvous_area_size)
#define scratch_buffer_off      (rendezvous_area_off - scratch_buffer_size)
#define vm_cache_off            (scratch_buffer_off - vm_cache_size)
#define vm_mbox_off             rendezvous_area_off
#define vm_params_off           (vm_mbox_off + vm_mbox_size)
#define serial_data_off         (vm_params_off + vm_params_size)

#define scratch_buffer          ((uint8_t *)(hub_memory_base + scratch_buffer_off))
#define vm_mbox                 ((uint32_t *)(hub_memory_base + vm_mbox_off))
#define vm_params               ((uint32_t *)(hub_memory_base + vm_params_off))
#define serial_data             ((FdSerial_t *)(hub_memory_base + serial_data_off))

/* ZPU's libgloss uses syscall IDs 1 to 18, we stay well away from that range */
#define SYS_cognew  8000
#define SYS_coginit 8001
#define SYS_cogstop 8002
#define SYS_mboxcmd 8003

extern int _syscall(int *foo, int ID, ...);

extern int errno;

void longfill(int* dst, int fill, int count);

void longmove(int* dst, int* src, int count);

int cognew(int* code, int* par);

void cogstop(int cog);

#ifdef __cplusplus
}
#endif

#endif
